module Language.VDM.Syntax where
 
data Document = Document [DefinitionBlock]
 
data DefinitionBlock = DB_TypeDefinitions TypeDefinitions
		     | DB_StateDefinition StateDefinition
		     | DB_ValueDefinitions ValueDefinitions
		     | DB_FunctionDefinitions FunctionDefinitions
		     | DB_OperationDefinitions OperationDefinitions
 
data TypeDefinitions = TypeDefinitions [TypeDefinition]
 
data TypeDefinition = UnTaggedTypeDef Identifier Type
				      (Maybe Invariant)
		    | TaggedTypeDef Identifier FieldList (Maybe Invariant)
 
data Type = BOOL
	  | NAT
	  | NATONE
	  | INTEGER
	  | RAT
	  | REAL
	  | CHAR
	  | TOKEN
	  | QuoteType QuoteLiteral
	  | CompositeType Identifier FieldList
	  | UnionType Type Type
	  | ProductType Type Type
	  | OptionalType Type
	  | SetType Type
	  | Seq0Type Type
	  | Seq1Type Type
	  | GeneralMapType Type Type
	  | InjectiveMapType Type Type
	  | FnType FunctionType
	  | TypeName Name
	  | TypeVar TypeVariableIdentifier
 
data FieldList = FieldList [Field]
 
data Field = Field (Maybe (Identifier)) Type
 
data FunctionType = PartialFnType DiscretionaryType Type
		  | TotalFnType DiscretionaryType Type
 
data DiscretionaryType = DiscretionaryType (Either Type ())
 
data TypeVariableIdentifier = TypeVarId Identifier
 
data Invariant = Invariant InvariantInitialFunction
 
data InvariantInitialFunction = InvInitFn Pattern Expression
 
data StateDefinition = StateDef Identifier FieldList
				(Maybe Invariant) (Maybe Initialization)
 
data Initialization = Initialization InvariantInitialFunction
 
data ValueDefinitions = ValueDefinitions [ValueDefinition]
 
data ValueDefinition = ValueDef Pattern (Maybe (Type)) Expression
 
data FunctionDefinitions = FunctionDefinitions [FunctionDefinition]
 
data FunctionDefinition = ExplFnDef Identifier
				    (Maybe TypeVariableList) FunctionType Identifier ParametersList
				    Expression (Maybe (Expression))
			| ImplFnDef Identifier (Maybe TypeVariableList) ParameterTypes
				    IdentifierTypePair (Maybe (Expression)) Expression
 
data TypeVariableList = TypeVarList [TypeVariableIdentifier]
 
data IdentifierTypePair = IdType Identifier Type
 
data ParameterTypes = ParameterTypes (Maybe PatternTypePairList)
 
data PatternTypePairList = PatTypePairList [PatternTypePair]
 
data PatternTypePair = PatTypePair PatternList Type
 
data ParametersList = ParametersList [Parameters]
 
data Parameters = Parameters (Maybe PatternList)
 
data OperationDefinitions = OperationDefinitions [OperationDefinition]
 
data OperationDefinition = ExplOprtDef Identifier OperationType
				       Identifier Parameters Statement (Maybe (Expression))
			 | ImplOprtDef Identifier ParameterTypes (Maybe IdentifierTypePair)
				       (Maybe Externals) (Maybe (Expression)) Expression
				       (Maybe Exceptions)
 
data OperationType = OpType DiscretionaryType DiscretionaryType
 
data Externals = Externals [VarInformation]
 
data VarInformation = VarInf Mode NameList (Maybe (Type))
 
data Mode = READ
	  | READWRITE
 
data Exceptions = Exceptions [Error]
 
data Error = Error Identifier Expression Expression
 
data ExpressionList = ExprList [Expression]
 
data Expression = LetExpr [LocalDefinition] Expression
		| LetBeSTExpr Bind (Maybe (Expression)) Expression
		| DefExpr [DefBind] Expression
		| IfExpr Expression Expression [ElseIfExpression] Expression
		| CasesExpr Expression CasesExpressionAlternatives
			    (Maybe (OthersExpression))
		| UnaryNumPlusExpr Expression
		| UnaryNumMinusExpr Expression
		| NumAbsExpr Expression
		| FloorExpr Expression
		| NotExpr Expression
		| SetCardExpr Expression
		| SetPowerExpr Expression
		| SetDistrUnionExpr Expression
		| SetDistrIntersectExpr Expression
		| SeqHeadExpr Expression
		| SeqTailExpr Expression
		| SeqLenExpr Expression
		| SeqElemsExpr Expression
		| SeqIndicesExpr Expression
		| SeqDistrConcExpr Expression
		| MapDomExpr Expression
		| MapRngExpr Expression
		| MapDistrMergeExpr Expression
		| MapInverseExpr Expression
		| BinaryNumPlusExpr Expression Expression
		| BinaryNumMinusExpr Expression Expression
		| NumPlusExpr Expression Expression
		| NumDivExpr Expression Expression
		| IntDivExpr Expression Expression
		| NumRemExpr Expression Expression
		| NumModExpr Expression Expression
		| NumLtExpr Expression Expression
		| NumLeExpr Expression Expression
		| NumGtExpr Expression Expression
		| NumGeExpr Expression Expression
		| EqExpr Expression Expression
		| NeExpr Expression Expression
		| OrExpr Expression Expression
		| AndExpr Expression Expression
		| ImplyExpr Expression Expression
		| EquivExpr Expression Expression
		| InSetExpr Expression Expression
		| NotInSetExpr Expression Expression
		| SubsetExpr Expression Expression
		| ProperSubsetExpr Expression Expression
		| SetUnionExpr Expression Expression
		| SetDifference Expression Expression
		| SetIntersect Expression Expression
		| SeqConc Expression Expression
		| MapOrSeqModExpr Expression Expression
		| MapMergeExpr Expression Expression
		| MapDomRestrToExpr Expression Expression
		| MapDomRestrByExpr Expression Expression
		| MapRngRestrToExpr Expression Expression
		| MapRngRestrByExpr Expression Expression
		| ComposeExpr Expression Expression
		| IterateExpr Expression Expression
		| QuantExpr QuantifiedExpression
		| IotaExpr Bind Expression
		| SetEnumeration (Maybe ExpressionList)
		| SetComprehension Expression BindList (Maybe (Expression))
		| SetRange Expression Expression
		| SeqEnumeration (Maybe ExpressionList)
		| SeqComprehension Expression SetBind (Maybe (Expression))
		| SubSequence Expression Expression Expression
		| MapEnumeration (Either () ([Maplet]))
		| MapComprehension Maplet BindList (Maybe (Expression))
		| TupleConstructor Expression ExpressionList
		| RecordConstructor Name (Maybe ExpressionList)
		| RecordModifier Expression [RecordModification]
		| Apply Expression (Maybe ExpressionList)
		| FieldSelect Expression Identifier
		| FctTypeInst Name [Type]
		| Lambda TypeBindList Expression
		| IsBoolExpr Expression
		| IsNatExpr Expression
		| IsNatOneExpr Expression
		| IsIntegerExpr Expression
		| IsRatExpr Expression
		| IsRealExpr Expression
		| IsCharExpr Expression
		| IsTokenExpr Expression
		| IsDefTypeExpr Identifier Expression
		| NameExpr Name
		| OldNameExpr OldName
		| LiteralExpr SymbolicLiteral
 
data DefBind = DefBind PatternBind Expression
 
data ElseIfExpression = ElsifExpr Expression Expression
 
data CasesExpressionAlternatives = CasesExprAlternatives [CasesExpressionAlternative]
 
data CasesExpressionAlternative = CaseAltn PatternList Expression
 
data OthersExpression = OthersExpr Expression
 
data QuantifiedExpression = AllExpr BindList Expression
			  | ExistsExpr BindList Expression
			  | ExistsUniqueExpr BindList Expression
 
data Maplet = Maplet Expression Expression
 
data RecordModification = RecordModification Identifier Expression
 
data Name = Name Identifier
 
data NameList = NameList [Name]
 
data OldName = OldName Identifier
 
data StateDesignator = StDesignatorName Name
		     | FieldRef StateDesignator Identifier
		     | MapOrSeqRef StateDesignator Expression
 
data Statement = LetStmt [LocalDefinition] Statement
	       | LetBeSTStmt Bind (Maybe (Expression)) Statement
	       | DefStmt [EqualsDefinition] Statement
	       | BlockStmt [DclStatement] [Statement]
	       | AssignStmt AssignStatement
	       | IfStmt Expression Statement [ElseifStatement] (Maybe (Statement))
	       | CasesStmt Expression CasesStatementAlternatives
			   (Maybe (OthersStatement))
	       | SeqForLoop PatternBind (Maybe ()) Expression Statement
	       | SetForLoop Pattern Expression Statement
	       | IndexForLoop Identifier Expression Expression
			      (Maybe (Expression)) Statement
	       | WhileLoop Expression Statement
	       | NonDetStmt [Statement]
	       | CallStmt CallStatement
	       | ReturnStmt (Maybe Expression)
	       | AlwaysStmt Statement Statement
	       | TrapStmt PatternBind Statement Statement
	       | RecTrapStmt Traps Statement
	       | ExitStmt (Maybe Expression)
	       | IdentStmt
 
data LocalDefinition = LocalDefinition (Either FunctionDefinition
					  ValueDefinition)
 
data EqualsDefinition = EqualsExprDef PatternBind Expression
		      | EqualsCallStmtDef PatternBind CallStatement
 
data DclStatement = DclStmt AssignmentDefinition
 
data AssignmentDefinition = AssignDefExpr Identifier Type
					  (Maybe (Expression))
			  | AssignDefCallStmt Identifier Type (Maybe (CallStatement))
 
data AssignStatement = AssignStmtExpr StateDesignator Expression
		     | AssignStmtCallStmt StateDesignator CallStatement
 
data ElseifStatement = ElsifStmt Expression Statement
 
data CasesStatementAlternatives = CasesStmtAlternatives [CasesStatementAlternative]
 
data CasesStatementAlternative = CasesStmtAltn PatternList
					       Statement
 
data OthersStatement = OthersStmt Statement
 
data CallStatement = Call Name (Maybe ExpressionList)
			  (Maybe (StateDesignator))
 
data Traps = Traps [Trap]
 
data Trap = Trap PatternBind Statement
 
data Pattern = PatternId (Either () Identifier)
	     | MatchValueExpr Expression
	     | MatchValueSymb SymbolicLiteral
	     | SetEnumPattern PatternList
	     | SetUnionPattern Pattern Pattern
	     | SeqEnumPattern PatternList
	     | SeqConcPattern Pattern Pattern
	     | TuplePattern Pattern PatternList
	     | RecordPattern Name (Maybe PatternList)
 
data PatternList = PatList [Pattern]
 
data PatternBind = PatternBind (Either Pattern Bind)
 
data Bind = Bind (Either SetBind TypeBind)
 
data SetBind = SetBind Pattern Expression
 
data TypeBind = TypeBind Pattern Type
 
data BindList = BindList [MultipleBind]
 
data MultipleBind = MultSetBind PatternList Expression
		  | MultTypeBind PatternList Type
 
data TypeBindList = TypeBindList [TypeBind]
 
data SymbolicLiteral = NumericLiteral NumericLiteral
		     | BooleanLiteral BooleanLiteral
		     | NilLiteral
		     | CharacterLiteral CharacterLiteral
		     | TextLiteral TextLiteral
		     | QuoteLiteral QuoteLiteral
 
data BooleanLiteral = TRUE
		    | FALSE
 
type NumericLiteral = String
 
type Numeral = String
 
type Exponent = String
 
type CharacterLiteral = String
 
type TextLiteral = String
 
type QuoteLiteral = String
 
type Identifier = String
 
type Character = String
 
type PlainLetter = String
 
type GreekLetter = String
 
type UpperGreekLetter = String
 
type LowerGreekLetter = String
 
type Digit = String
 
type DelimiterCharacter = String
 
type OtherCharacter = String
 
type Separator = String